function [beta,data,stats]=fixed(data,dep,cov,id,output,cluster)
%Estimates models with many fixed effects
%fixed(data,dep,cov,id,output,cluster), where data is the data matrix. dep and cov
%specify the column number of the dependent and independent variable. dep
%must be a scalar, cov can be a vector. cov can also be [], indicating no
%covariates. In this case the fixed effects are identical in the ME and
%TWFE model, so the ME model is estimated. id is a vector specifying the
%identifiers for the fixed effects. The first element should be the larger
%set of fixed effects, the second the smaller. An optional third element
%can indicate the location of an identifier for the interaction between the
%first two fixed effects. Note that if no element is supplied, the TWFE model
%is estimated. If it is specified, the two other effects have to be constant 
%within each group defined by it or the results will not be valid. 
%Output defines what the function returns: 'beta' calculates slopes only, 
%'coef' returns the slopes and a structure 'data' containing the IDs, estimates 
%of the fixed effects and the connected group (Abowd et al 2002) they belong to. 
%'dataset' returns a matrix containing the IDs, the estimated fixed effects and the
%connected group the observation belongs to. Only specifying one output element is 
%equivalent to 'beta'.
%The mean of the first fixed effect is normalized to zero in each connected group 
%to achieve identification.
%cluster is an optional argument that can give the column numbers of up to two vector(s) 
%of id variables in the data at which level the errors should be clustered when computing 
%the variance matrix of beta. It can also be 'het' in which case a heteroskedasticity consistent 
%variance matrix is returned or 'resid' in which case the vector of residuals is returned 
%instead of the variance matrix of beta. 'none' is the default and assumes errors to be iid.

%call appropriate function
if nargin>6 || nargin<4
    error('Fixed:InArg','Wrong Number of Input arguments')
end

switch nargin
    case 4
        output='coef';
        cluster='none';
    case 5
        cluster='none';
end

if nargout>1 && strcmp(output,'beta')==1
    error('Fixed:OutArg','Too many output arguments for option "beta"')
end

if isempty(cov)==1
    warning('Fixed:NoCov','No covariates specified, estimating match effects model without covariates')
    if length(id)==2
        warning('Fixed:NoMID','No match identifier provided, using unique combinations of IDs')
        [a ,~, c]=unique(data(:,id),'rows');
        b=1:size(a,1);
        id=[id size(data,2)+1];
        data(:,id(3))=b(c);
        clear a b c
    end
end
        
if length(id)==2
    if strcmp(output,'beta')==1 || nargout==1
        if isnumeric(cluster)==1
            fprintf(1,'Estimating TWFE model with %i covariates, output option beta and %i-way clustering \n',length(cov),length(cluster))
        else
            fprintf(1,'Estimating TWFE model with %i covariates, output option beta and clustering option %s \n',length(cov),cluster)
        end
        beta=fenomatch(data,dep,cov,id(1),id(2),'beta',cluster);
    elseif nargout==2 && strcmp(output,'beta')==0
        if isnumeric(cluster)==1
            fprintf(1,'Estimating TWFE model with %i covariates, output option %s and %i-way clustering \n',length(cov),output,length(cluster))
        else
            fprintf(1,'Estimating TWFE model with %i covariates, output option %s and clustering option %s \n',length(cov),output,cluster)
        end
        [beta,data]=fenomatch(data,dep,cov,id(1),id(2),output,cluster);
    elseif nargout==3 && strcmp(output,'beta')==0;
        if isnumeric(cluster)==1
            fprintf(1,'Estimating TWFE model with %i covariates, output option %s and %i-way clustering \n',length(cov),output,length(cluster))
        else
            fprintf(1,'Estimating TWFE model with %i covariates, output option %s and clustering option %s \n',length(cov),output,cluster)
        end        
        [beta,data,stats]=fenomatch(data,dep,cov,id(1),id(2),output,cluster);
    else error('Fixed:OutArg','Wrong Number of Output arguments')
    end
elseif length(id)==3
    if strcmp(output,'beta')==1 || nargout==1
        if isnumeric(cluster)==1
            fprintf(1,'Estimating Match Effects model with %i covariates, output option beta and %i-way clustering \n',length(cov),length(cluster))
        else
            fprintf(1,'Estimating Match Effects model with %i covariates, output option beta and clustering option %s \n',length(cov),cluster)
        end
        beta=fixedeffects(data,dep,cov,id(1),id(2),id(3),output,cluster);
    elseif nargout==2 && strcmp(output,'beta')==0
        if isnumeric(cluster)==1
            fprintf(1,'Estimating Match Effects model with %i covariates, output option %s and %i-way clustering \n',length(cov),output,length(cluster))
        else
            fprintf(1,'Estimating Match Effects model with %i covariates, output option %s and clustering option %s \n',length(cov),output,cluster)
        end
        [beta,data]=fixedeffects(data,dep,cov,id(1),id(2),id(3),output,cluster);
    elseif nargout==3 && strcmp(output,'beta')==0;
        if isnumeric(cluster)==1
            fprintf(1,'Estimating Match Effects model with %i covariates, output option %s and %i-way clustering \n',length(cov),output,length(cluster))
        else
            fprintf(1,'Estimating Match Effects model with %i covariates, output option %s and clustering option %s \n',length(cov),output,cluster)
        end        
        [beta,data,stats]=fixedeffects(data,dep,cov,id(1),id(2),id(3),output,cluster);
    else error('Wrong Number of Output arguments')
    end
else error('Fixed:ID','Wrong number of ID variables') 
end
end

function [beta,data,stats]=fenomatch(data,dep,cov,id1,id2,output,cluster)
%Estimates models with many fixed effects
%fixedeffects(data,dep,cov,id1,id2,output), where data is the data matrix,
%dep, cov, id1, id2, id3 specify the column number of the dependent
%variable, the independent variables and the individual, firm and spell
%identifiers of which only cov can be a vector.

%create IDs that are consecutive integers
%individual ids
data=sortrows(data,id1);
iv = diff([ 0 find(data(1:end-1,id1) ~= data(2:end,id1))' length(data(:,id1)) ]);
i = cumsum([ 1 iv ]);
j = zeros(1, i(end)-1);
j(i(1:end-1)) = 1;
i_id=size(data,2)+1;
data(:,i_id) = cumsum(j);
clear i j

%firm ids
data=sortrows(data,id2);
fv = diff([ 0 find(data(1:end-1,id2) ~= data(2:end,id2))' length(data(:,id2)) ]);
i = cumsum([ 1 fv ]);
j = zeros(1, i(end)-1);
j(i(1:end-1)) = 1;
f_id=size(data,2)+1;
data(:,f_id) = cumsum(j);
clear i j

%identify connected groups
g_id=size(data,2)+1;
i=0;
data(:,g_id)=0;
data=sortrows(data,[i_id f_id]);

while nnz(data(:,g_id))<size(data,1),
    i=i+1;
    b=zeros(max(data(:,i_id)),1);
    d=zeros(max(data(:,f_id)),1);
    g=zeros(size(data,1),1);
    alag=[];
    g(data(:,f_id)==data(find(data(:,g_id)==0,1),f_id))=i;
    a=unique(data(g==i,i_id));

while length(a)>length(alag),
    b(a)=1;
    g=(b(data(:,i_id))==1) | (g==1);
    alag=a;
    c=unique(data(g==1,f_id));
    d(c)=1;
    g=(d(data(:,f_id))==1) | (g==1);
    a=unique(data(g==1,i_id));
end
data(:,g_id)=data(:,g_id)+g.*i;
end
clear a alag b c d g i

%obtain beta.ols
%WK transformation
nf=length(fv);
K=sparse(data(:,i_id),1:size(data,1),1)*sparse(1:size(data,1),data(:,f_id),1);
if max(data(:,g_id))>1
    save('k.mat','K')
end

%firm sums of y, cov
data=sortrows(data,[f_id i_id]);
M=sparse(data(:,f_id),1:size(data,1),1);
fs=M*data(:,[dep,cov]);
clear M

%individual means of y, cov
data=sortrows(data,[i_id f_id]);
M=sparse(data(:,i_id),1:size(data,1),1./iv(data(:,i_id))');
im=M*data(:,[dep,cov]);
ni=length(iv);
clear M

%get residuals of restricted models for F-test
if nargout==3
    tss=data(:,dep);
    r=regstats(data(:,dep),data(:,cov),'linear','r');
    rssall=r.r;
    fm=fs./fv(ones(size(fs,2),1),:)';
    r=regstats(data(:,dep)-fm(data(:,f_id),1),data(:,cov)-fm(data(:,f_id),2:end),'linear','r');
    rssfe1=r.r;
    r=regstats(data(:,dep)-im(data(:,i_id),1),data(:,cov)-im(data(:,i_id),2:end),'linear','r');
    rssfe2=r.r;
    clear r
end

%solve for every group
data(:,[dep,cov])=data(:,[dep,cov])-im(data(:,i_id),:);
dec=zeros(max(data(:,g_id)),1);
for i=1:max(data(:,g_id)),
    [a,~,c]=unique(data(data(:,g_id)==i,f_id));
    [b,~,d]=unique(data(data(:,g_id)==i,i_id));
    if i>1 
        load('k.mat','K');
    end
    K=K(b,a);
    nfg=length(a);
    nig=length(b);
    if nfg>1
        T=sparse(1:nfg,1:nfg,fv(a)')-K'*sparse(1:nig,1:nig,1./iv(b)')*K;
        M=fs(a,:)-K'*im(b,:);
        L=genvarname(strcat('L',num2str(i)));
        try
            lastwarn('');
            eval([L '=chol(T);']);
            if ~isempty(lastwarn) 
                fprintf(1,'Warning above is from Cholesky decomposition for group %i',i)
            end
            if max(data(:,g_id))>1          
                if i==1,
                    save('l.mat',strcat('L',num2str(i)))
                else
                    save('l.mat',strcat('L',num2str(i)),'-append')
                end
            end
            y=eval(L)'\M;
            T=eval(L)\y;
        catch ME
            if strcmp(ME.identifier,'MATLAB:posdef')==1,
                warning('Fixed:MatDec','Cholesky decomposition failed for group %i because matrix is not pos. def at double precision. Attempting LU decomposition.', i)
            else
                warning('Fixed:MatDec','Cholesky decomposition failed for group %i, attempting LU decomposition. Error message: "%s"',i,ME.message)
            end
            dec(i)=1;
            U=genvarname(strcat('U',num2str(i)));
            lastwarn('');
            eval(['[' L ',' U '] =lu(T);']);
            if ~isempty(lastwarn) 
                fprintf(1,'Warning above is from LU decomposition for group %i',i)
            end
                if max(data(:,g_id))>1          
                    if i==1,
                        save('l.mat',strcat('L',num2str(i)))
                        save('u.mat',strcat('U',num2str(i)))
                    else
                        save('l.mat',strcat('L',num2str(i)),'-append')
                        if sum(dec)==1
                            save('u.mat',strcat('U',num2str(i)))
                        else
                            save('u.mat',strcat('U',num2str(i)),'-append')
                        end
                    end
                end
            y = eval(L)\M;
            T = eval(U)\y;
        end
        clear M y
        %transform data
        b=sparse(1:nig,1:nig,1./iv(b)')*K*T;
        data(data(:,g_id)==i,[dep,cov])=data(data(:,g_id)==i,[dep,cov])+b(d,:)-T(c,:);
        if max(data(:,g_id))>1
            eval(['clear ' L]);
            if dec(i)==1
                eval(['clear ' U]);
                clear U
            end
            clear K
        end
        clear T L
    else
        data(data(:,g_id)==i,[dep,cov])=data(data(:,g_id)==i,[dep,cov])-im(d,:);
    end
end
clear a b c d nfg nig 

%adjust numbers for dropped non-identified effects
a=~isnan(data(:,dep));
no=sum(a);
nf=nf-length(unique(data(~a,f_id)));
ni=ni-length(unique(data(~a,i_id)));
%run ols
if nargout==3
    if strcmp(cluster,'none')==1,
        beta=regstats(data(:,dep),data(:,cov),'linear',{'beta','covb','r'});
        beta.covb=beta.covb(2:end,2:end)*((no-size(im,2))/(no-size(im,2)-nf-ni+2));
    elseif strcmp(cluster,'het')==1,
        beta=regstats(data(:,dep),data(:,cov),'linear',{'beta','r'});
        beta.covb=covmat(beta.r,data(:,cov),cluster,size(im,2)+nf+ni-2);
    elseif strcmp(cluster,'resid')==1,
        beta=regstats(data(:,dep),data(:,cov),'linear',{'beta','r'});
        beta.covb=beta.r;
    else
        beta=regstats(data(:,dep),data(:,cov),'linear',{'beta','r'});
        beta.covb=covmat(beta.r,data(:,cov),data(:,cluster),size(im,2)+nf+ni-2);
    end
    %run restricted regressions
    rssall=rssall(a)'*rssall(a);
    tss=no*var(tss(a),1);
    rssfe1=rssfe1(a)'*rssfe1(a);
    rssfe2=rssfe2(a)'*rssfe2(a);
    %calculate statistics
    rss=beta.r(a)'*beta.r(a);
    beta=struct('beta',beta.beta(2:end),'covb',beta.covb);
    stats=struct('n',no,'ni',ni,'nf',nf,'TSS',tss,'RSS',rss,'R2',1-rss/tss,'fbeta',(beta.beta'/beta.covb)*beta.beta/size(im,2));
    stats.pbeta=1-fcdf(stats.fbeta,length(beta.beta),no-length(beta.beta)-ni-nf+1);
    stats.fall=((rssall-rss)/(nf+ni-2))/(rss/(no-size(im,2)-nf-ni+2));
    stats.pall=1-fcdf(stats.fall,nf+ni-2,no-size(im,2)-nf-ni+2);
    stats.ffe1=((rssfe1-rss)/(ni-1))/(rss/(no-size(im,2)-nf-ni+2));
    stats.pfe1=1-fcdf(stats.ffe1,ni-1,no-size(im,2)-nf-ni+2);
    stats.ffe2=((rssfe2-rss)/(nf-1))/(rss/(no-size(im,2)-nf-ni+2));
    stats.pfe2=1-fcdf(stats.ffe2,nf-1,no-size(im,2)-nf-ni+2);
    clear tss rss rssall rssfe1 rssfe2
else
    if strcmp(cluster,'none')==1,
        beta=regstats(data(:,dep),data(:,cov),'linear',{'beta','covb'});
        beta.covb=beta.covb(2:end,2:end)*((no-size(im,2))/(no-size(im,2)-nf-ni+2));
    elseif strcmp(cluster,'het')==1,
        beta=regstats(data(:,dep),data(:,cov),'linear',{'beta','r'});
        beta.covb=covmat(beta.r,data(:,cov),cluster,size(im,2)+nf+ni-2);
    elseif strcmp(cluster,'resid')==1,
        beta=regstats(data(:,dep),data(:,cov),'linear',{'beta','r'});
        beta.covb=beta.r;
    else
        beta=regstats(data(:,dep),data(:,cov),'linear',{'beta','r'});
        beta.covb=covmat(beta.r,data(:,cov),data(:,cluster),size(im,2)+nf+ni-2);
    end
    beta=struct('beta',beta.beta(2:end),'covb',beta.covb);    
end

clear cluster no a

if strcmp(output,'beta')==1
    clear data im a fv fm iv dep nf ni f_id fs i_id id1 id2
    return
end

clear cov dep
%adjust means
im=im(:,1)-im(:,2:end)*beta.beta;

if nargout==3
    fm=fm(:,1)-fm(:,2:end)*beta.beta;
else
    fm=fs(:,1)./fv'-(fs(:,2:end)./fv(ones(size(fs,2)-1,1),:)')*beta.beta;
end
data=data(:,[id1 id2 i_id f_id g_id]);
clear fs id1 id2 i_id f_id g_id

%get firm and individual fixed effects
fe1=zeros(ni,2);
fe2=zeros(nf,2);
clear nf ni
for i=1:max(data(:,5)),
    a=unique(data(data(:,5)==i,4));
    b=unique(data(data(:,5)==i,3));
    if length(a)==1
        fe2(a,1)=fm(a);
        fe2(a,2)=i;
        fe1(b,1)=im(b)-fm(a);
        fe1(b,2)=i;
    else
        L=genvarname(strcat('L',num2str(i)));
        if max(data(:,5))>1
            load('k.mat','K')
            load('l.mat',strcat('L',num2str(i)));
            if dec(i)==1
                load('u.mat',strcat('U',num2str(i)));
            end
        end
        K=K(b,a);
        M=fm(a).*fv(a)'-K'*im(b);
        if dec(i)==0
            y=eval(L).'\M;
            fe2(a,1)=eval(L)\y;
        elseif dec(i)==1
            U=genvarname(strcat('U',num2str(i)));
            y=eval(L)\M;
            fe2(a,1)=eval(U)\y;
            eval(['clear ' U]);
        end
        fe2(a,2)=i;
        fe1(b,1)=im(b)-K*fe2(a,1)./iv(b)';
        fe1(b,2)=i;
        %impose normalization
        fe2(a,1)=fe2(a,1)+mean(fe1(b,1));
        fe1(b,1)=fe1(b)-mean(fe1(b,1));
    end
    eval(['clear ' L]);
end
clear L M a b fm fv K im iv y
if max(data(:,5))>1
    delete('k.mat','l.mat')
    if max(dec)==1
        delete('u.mat')
    end
end
clear dec

if strcmp(output,'coef')==1
    data=struct('fe1',unique(data(:,1)),'fe2',unique(data(:,2)));
    data.fe2(:,2:3)=fe2;
    clear fe2
    data.fe1(:,2:3)=fe1;
    clear fe1
    return
else
    data(:,3) = fe1(data(:,3),1);
    clear fe1
    data(:,[4,5]) = fe2(data(:,4),:);
    clear fe2
end
end

function [beta,data,stats]=fixedeffects(data,dep,cov,id1,id2,id3,output,cluster)
%Estimates models with many fixed effects
%fixedeffects(data,dep,cov,id1,id2,id3,output), where data is the data matrix,
%dep, cov, id1, id2, id3 specify the column number of the dependent
%variable, the independent variables and the individual, firm and spell
%identifiers of which only cov can be a vector.

%create IDs that are consecutive integers
data=sortrows(data,id1);
iv = diff([ 0 find(data(1:end-1,id1) ~= data(2:end,id1))' length(data(:,id1)) ]);
i = cumsum([ 1 iv ]);
j = zeros(1, i(end)-1);
j(i(1:end-1)) = 1;
i_id=size(data,2)+1;
data(:,i_id) = cumsum(j);
ni=length(iv);
clear i j

data=sortrows(data,id2);
fv = diff([ 0 find(data(1:end-1,id2) ~= data(2:end,id2))' length(data(:,id2)) ]);
i = cumsum([ 1 fv ]);
j = zeros(1, i(end)-1);
j(i(1:end-1)) = 1;
f_id=size(data,2)+1;
data(:,f_id) = cumsum(j);
nf=length(fv);
clear i j

data=sortrows(data,id3);
sv = diff([ 0 find(data(1:end-1,id3) ~= data(2:end,id3))' length(data(:,id3)) ]);
i = cumsum([ 1 sv ]);
j = zeros(1, i(end)-1);
j(i(1:end-1)) = 1;
s_id=size(data,2)+1;
data(:,s_id) = cumsum(j);
ns=length(sv);
clear i j

%identify connected groups
g_id=size(data,2)+1;
i=0;
data(:,g_id)=0;
data=sortrows(data,[i_id f_id]);

while nnz(data(:,g_id))<size(data,1),
    i=i+1;
    b=zeros(max(data(:,i_id)),1);
    d=zeros(max(data(:,f_id)),1);
    g=zeros(size(data,1),1);
    alag=[];
    g(data(:,f_id)==data(find(data(:,g_id)==0,1),f_id))=i;
    a=unique(data(g==i,i_id));

while length(a)>length(alag),
    b(a)=1;
    g=(b(data(:,i_id))==1) | (g==1);
    alag=a;
    c=unique(data(g==1,f_id));
    d(c)=1;
    g=(d(data(:,f_id))==1) | (g==1);
    a=unique(data(g==1,i_id));
end
data(:,g_id)=data(:,g_id)+g.*i;
end
clear a alag b c d g i

%get rss for ftests
if nargout==3
    r=regstats(data(:,dep),data(:,cov),'linear',{'r','rsquare'});
    rssall=r.r'*r.r;
    tss=rssall/(1-r.rsquare);
    clear r
end

no=length(data(:,dep));
%case with covariates
if isempty(cov)==0
    %get and subtract spell means
    covar=data(:,cov);
    x = sv(data(:,s_id));
    M=sparse(data(:,s_id),1:size(data,1),1./x');
    sm=M*data(:,[dep,cov]);
    l=size(data,2)+1;
    data(:,[l,cov])=data(:,[dep,cov])-sm(data(:,s_id),:);
    clear x M i j sv
    
    %get beta
    if nargout==3
        if strcmp(cluster,'none')==1,
            beta=regstats(data(:,l),data(:,cov),'linear',{'beta','covb','r'});
            beta.covb=beta.covb(2:end,2:end)*((no-length(cov)-1)/(no-length(cov)-ns));
        elseif strcmp(cluster,'het')==1,
            beta=regstats(data(:,l),data(:,cov),'linear',{'beta','r'});
            beta.covb=covmat(beta.r,data(:,cov),'het',length(cov)+ns);
        elseif strcmp(cluster,'resid')==1,
            beta=regstats(data(:,l),data(:,cov),'linear',{'beta','r'});
            beta.covb=beta.r;
        else
            beta=regstats(data(:,l),data(:,cov),'linear',{'beta','r'});
            beta.covb=covmat(beta.r,data(:,cov),data(:,cluster),length(cov)+ns);
        end
        rss=beta.r'*beta.r;
        beta=struct('beta',beta.beta(2:end),'covb',beta.covb);
        stats=struct('n',no,'ni',ni,'nf',nf,'nm',ns,'TSS',tss,'RSS',rss,'R2',1-rss/tss,'fbeta',(beta.beta'/beta.covb)*beta.beta/length(beta.beta));
        stats.pbeta=1-fcdf(stats.fbeta,length(beta.beta),no-length(beta.beta)-ni-nf+2);
        stats.fall=((rssall-rss)/(ns-1))/(rss/(no-length(cov)-ns));
        stats.pall=1-fcdf(stats.fall,ns-1,no-length(cov)-ns);
        clear rss rssall tss
    else
        if strcmp(cluster,'none')==1,
            beta=regstats(data(:,l),data(:,cov),'linear',{'beta','covb'});
            beta.covb=beta.covb(2:end,2:end)*((no-length(cov)-1)/(no-length(cov)-ns));
        elseif strcmp(cluster,'het')==1,
            beta=regstats(data(:,l),data(:,cov),'linear',{'beta','r'});
            beta.covb=covmat(beta.r,data(:,cov),'het',length(cov)+ns);
        elseif strcmp(cluster,'resid')==1,
            beta=regstats(data(:,l),data(:,cov),'linear',{'beta','r'});
            beta.covb=beta.r;
        else
            beta=regstats(data(:,l),data(:,cov),'linear',{'beta','r'});
            beta.covb=covmat(beta.r,data(:,cov),data(:,cluster),length(cov)+ns);
        end
        beta=struct('beta',beta.beta(2:end),'covb',beta.covb);    
    end
    %adjust y
    data(:,dep)=data(:,dep)-covar*beta.beta;
    clear l covar ns
    %subtract xb from spell means
    sm=sm(:,1)-sm(:,2:end)*beta.beta;
else
%no covariates case
%get statistics and rss for ftests
    if nargout==3
        rssall=no*var(data(:,dep),1);
        tss=rssall;
        clear r
    end
    %get spell means, rss
    data=sortrows(data,[i_id f_id]);
    x = sv(data(:,s_id));
    M=sparse(data(:,s_id),1:size(data,1),1./x');
    sm=M*data(:,dep);
    clear x M
    if strcmp(cluster,'resid')==1        
        beta=struct('beta','none','covb',data(:,dep)-sm(data(:,s_id)));
    else beta=struct('beta','none','covb','none');
    end
    if nargout==3,
        rss=no*var(data(:,dep)-sm(data(:,s_id),1),1);
        stats=struct('n',no,'ni',ni,'nf',nf,'nm',ns,'TSS',tss,'RSS',rss,'R2',1-rss/tss);
        stats.fall=((rssall-rss)/(ns-1))/(rss/(no-ns));
        stats.pall=1-fcdf(stats.fall,ns-1,no-ns);
    end
end
    
clear cluster

if strcmp(output,'beta')==1
    clear data M fv iv no dep cov nf ni f_id i_id s_id id1 id2 id3 l
    return
end

%get means
%get individual means
data=sortrows(data,[i_id f_id]);
x = iv(data(:,i_id));
M=sparse(data(:,i_id),1:size(data,1),1./x');
im=M*data(:,dep);
clear M x

%get firm means
data=sortrows(data,[f_id s_id]);
x = fv(data(:,f_id));
M=sparse(data(:,f_id),1:size(data,1),1./x');
fm=M*data(:,dep);
clear M x

if isempty(cov)==1
    %get rss for individual FE only
    rssfe2=no*var(data(:,dep)-im(data(:,i_id)),1);
    rssfe1=no*var(data(:,dep)-fm(data(:,f_id)),1);
end

%get K
data=sortrows(data,[i_id f_id]);
K=sparse(data(:,i_id),1:size(data,1),1)*sparse(1:size(data,1),data(:,f_id),1);
if max(data(:,g_id))>1
    save('k.mat','K')
end
data=data(:,[id1 id2 id3 i_id f_id s_id g_id]);
clear dep i_id f_id s_id g_id id1 id2 id3

%get firm and individual fixed effects
fe1=zeros(ni,2);
fe2=zeros(nf,2);
if isempty(cov)==0
    clear ni nf no
end
for i=1:max(data(:,7)),
    a=unique(data(data(:,7)==i,5));
    b=unique(data(data(:,7)==i,4));
    if i>1 
        load('k.mat','K');
    end
    K=K(b,a);
    nfg=length(a);
    nig=length(b);
    if nfg==1
        fe2(a,1)=fm(a);
        fe2(a,2)=i;
        fe1(b,1)=im(b)-fm(a);
        fe1(b,2)=i;
    else
        M=fm(a).*fv(a)'-K'*im(b);
        lastwarn('');
        fe2(a,1)=(sparse(1:nfg,1:nfg,fv(a)')-K'*sparse(1:nig,1:nig,1./iv(b)')*K)\M;
        if ~isempty(lastwarn)
            fprintf(1,'Warning above is from group %i',i)
        end
        fe2(a,2)=i;
        fe1(b,1)=im(b)-K*fe2(a,1)./iv(b)';
        fe1(b,2)=i;
        %impose normalization
        fe2(a,1)=fe2(a,1)+mean(fe1(b,1));
        fe1(b,1)=fe1(b,1)-mean(fe1(b,1));
    end
end
clear L M a b fm fv nig nfg K im iv
if max(data(:,7))>1
    delete('k.mat')
end

if strcmp(output,'coef')==1
    data=sortrows(data,[4 5]);
    me=sm(data(:,6))-fe1(data(:,4),1)-fe2(data(:,5),1);
    clear sm output
    if isempty(cov)==1 && nargout==3
       rssTW=me'*me+rss;
    end 
    data=struct('fe1',unique(data(:,1)),'fe2',unique(data(:,2)),'me',unique([data(:,1:3) me],'rows'));
    clear me
    data.fe2(:,2:3)=fe2;
    clear fe2
    data.fe1(:,2:3)=fe1;
    clear fe1
else
    data(:,4) = fe1(data(:,4),1);
    clear fe1 output
    data(:,[5,7]) = fe2(data(:,5),:);
    clear fe2
    data(:,6) = sm(data(:,6))-data(:,4)-data(:,5);
    clear sm
    if isempty(cov)==1 && nargout==3
       rssTW=data(:,6)'*data(:,6)+rss;
    end 
end

if isempty(cov)==1 && nargout==3
       stats.rssTW=rssTW;
       stats.R2TW=1-rssTW/tss;
       stats.fme=((rssTW-rss)/(ns-nf-ni))/(rss/(no-ns));
       stats.pme=1-fcdf(stats.fme,ns-nf-ni,no-ns);
       stats.ffe=((rssall-rssTW)/(ni+nf-1))/(rssTW/(no-ni-nf+1));
       stats.pfe=1-fcdf(stats.ffe,ni+nf-1,no-ni-nf+1);
       stats.ffe1=((rssfe1-rssTW)/(ni-1))/(rssTW/(no-ni-nf+1));
       stats.pfe1=1-fcdf(stats.ffe1,ni-1,no-ni-nf+1);
       stats.ffe2=((rssfe2-rssTW)/(nf-1))/(rssTW/(no-ni-nf+1));
       stats.pfe2=1-fcdf(stats.ffe2,nf-1,no-ni-nf+1);
       clear nf ni no ns rssTW rssfe1 rssfe2 rss rssall tss
end
clear cov 
end

function covb=covmat(resid,X,cluster,df)
%estimates the variance matrix of beta if errors are not iid
%covmat(resid,X,cluster), where resid are the residuals from the regression, 
%X is the matrix of independent variables used in the regression, and
%cluster is the vector/matrix of id variables on which errors are clustered.
%If cluster is 'het', heteroskedasticity consistent SEs are computed. As of
%now, this only works for up to two id variables. ID variables need to be
%consecutive integers, but that is easy to change.
%df is optional and can give the number of degrees of freedom used by the
%model if the degrees of freedom correction is desired. This correction
%only needs to be done to account for the number of parameters in the
%model, a finite sample correction for the number of clusters is done
%automatically.

%setup
if nargin==3, df=0;
end
a=~isnan(resid);
resid=resid(a);
X=X(a,:);
if strcmp(cluster,'het')==0
    cluster=cluster(a,:);
end
N=length(resid);
xx=X'*X;

%create id for intersection of clusters
if size(cluster,2)==2,
[d ,~, f]=unique(cluster,'rows');
e=1:size(d,1);
cluster(:,size(cluster,2)+1)=e(f);
clear d e f
end

%compute matrices
if strcmp(cluster,'het')==1
    covb=(N/(N-df))*xx\(X'*sparse(1:N,1:N,resid.^2)*X)/xx;
    clear cluster N xx df X resid
    return
else
    a=size(X,2);
    b=1;
    X(:,a+1)=resid;
    xeex=zeros(a);
    matlabpool
    for j=1:size(cluster,2),
        if j==3, b=-1;
        end
        if j>1, cluster=sortrows(cluster,j-1);
        end
        X(:,a+2)=cluster(:,j);
        X=sortrows(X,a+2);
        c=diff([0 find(X(1:end-1,a+2) ~= X(2:end,a+2))' N]);
        ncl=length(c);
        submats=mat2cell(X,c,a+2);
        parfor i=1:ncl,
            e=submats{i};
            xeex=xeex+b*(ncl/(ncl-1)).*(e(:,1:a)'*e(:,a+1)*e(:,a+1)'*e(:,1:a));
        end
    end
    matlabpool close
    clear ncl a b c e X cluster submats
    covb=(N/(N-df))*xx\xeex/xx;
    clear N xx xeex df
end
end
